#define _CRT_SECURE_NO_WARNINGS
#include "testlib.h"
#include <unordered_map>
#include <unordered_set>
#include <algorithm>
#include <iostream>
#include <cstring>
#include <cassert>
#include <complex>
#include <chrono>
#include <random>
#include <bitset>
#include <cstdio>
#include <vector>
#include <string>
#include <stack>
#include <tuple>
#include <queue>
#include <ctime>
#include <cmath>
#include <list>
#include <map>
#include <set>

using namespace std;

struct Test {
  int n;
  vector < pair<int, int> > cactus;
};

int main(int argc, char **argv) {
  registerGen(argc, argv, 1);

  int start = atoi(argv[1]);

  vector <Test> tests;

  auto ShuffleCactus = [&](vector < pair<int, int> > &cactus) {
    for (int i = 0; i < (int)cactus.size(); i++) {
      if (rnd.next(2)) {
        swap(cactus[i].first, cactus[i].second);
      }
    }
    shuffle(cactus.begin(), cactus.end());
  };

  auto Add = [&](Test test) {
    ShuffleCactus(test.cactus);
    tests.push_back(test);
  };

  auto GenerateCycleLengths = [&](int odd_min, int odd_max, int odd_cnt, int even_min, int even_max, int even_cnt) {
    vector <int> lengths;
    for (int i = 1; i <= odd_cnt; i++) {
      lengths.push_back(rnd.next((odd_min - 1) / 2, (odd_max - 1) / 2) * 2 + 1);
    }
    for (int i = 1; i <= even_cnt; i++) {
      lengths.push_back(rnd.next(even_min / 2, even_max / 2) * 2);
    }
    shuffle(lengths.begin(), lengths.end());

    return lengths;
  };

  auto GenerateRandomOrder = [&](int n) {
    vector <int> vertices;
    for (int i = 1; i <= n; i++) {
      vertices.push_back(i);
    }
    shuffle(vertices.begin(), vertices.end());
    return vertices;
  };

  auto GenerateRandomCactus = [&](vector <int>& lengths) {
    int n = 1;
    for (int len: lengths) {
      n += len - 1;
    }

    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> availables = {vertices[0]};

    int it = 0;

    Test test;
    test.n = n;
    auto& cactus = test.cactus;

    for (int len: lengths) {
      vector <int> cycle = { availables[ rnd.next(0, (int)availables.size() - 1) ] };
      for (int j = 1; j <= len - 1; j++) {
        ++it;
        cycle.push_back(vertices[it]);
        availables.push_back(vertices[it]);
      }

      for (int i = 0; i < len; i++) {
        cactus.emplace_back(cycle[i], cycle[(i + 1) % len]);
      }
    }

    return test;
  };

  auto GenerateChainCactus = [&](vector <int>& lengths) {
    int n = 1;
    for (int len: lengths) {
      n += len - 1;
    }

    vector <int> vertices = GenerateRandomOrder(n);

    int current_root = vertices[0];
    int it = 0;

    Test test;
    test.n = n;
    auto& cactus = test.cactus;

    for (int len: lengths) {
      vector <int> cycle = {current_root};
      for (int j = 1; j <= len - 1; j++) {
        ++it;
        cycle.push_back(vertices[it]);
      }

      for (int i = 0; i < len; i++) {
        cactus.emplace_back(cycle[i], cycle[(i + 1) % len]);
      }

      current_root = cycle[ rnd.next(1, len - 1) ];
    }

    return test;
  };

  auto GenerateStarCactus = [&](vector <int>& lengths) {
    int n = 1;
    for (int len: lengths) {
      n += len - 1;
    }

    vector <int> vertices = GenerateRandomOrder(n);

    int root = vertices[0];
    int it = 0;

    Test test;
    test.n = n;
    auto& cactus = test.cactus;

    for (int len: lengths) {
      vector <int> cycle = {root};
      for (int j = 1; j <= len - 1; j++) {
        ++it;
        cycle.push_back(vertices[it]);
      }

      for (int i = 0; i < len; i++) {
        cactus.emplace_back(cycle[i], cycle[(i + 1) % len]);
      }
    }

    return test;
  };

  auto GenerateGoodRandomCactus = [&](vector <int>& lengths) {
    int sz = ((int)lengths.size() + 1) / 2;

    int n = 1;
    for (int len: lengths) {
      n += len - 1;
    }

    vector <int> vertices = GenerateRandomOrder(n);
    vector <int> parent = {vertices[0]};

    int current_root = vertices[0];
    int it = 0;

    Test test;
    test.n = n;
    auto& cactus = test.cactus;

    for (int i = 0; i < sz; i++) {
      int len = lengths[i];

      vector <int> cycle = {current_root};
      for (int j = 1; j <= len - 1; j++) {
        ++it;
        cycle.push_back(vertices[it]);
        parent.push_back(vertices[it]);
      }

      for (int j = 0; j < len; j++) {
        cactus.emplace_back(cycle[j], cycle[(j + 1) % len]);
      }

      current_root = cycle[ rnd.next(1, len - 1) ];
    }

    for (int i = sz; i < (int)lengths.size(); i++) {
      int len = lengths[i];

      vector <int> cycle = { parent[ rnd.next(0, (int)parent.size() - 1) ] };
      for (int j = 1; j <= len - 1; j++) {
        ++it;
        cycle.push_back(vertices[it]);
      }

      for (int j = 0; j < len; j++) {
        cactus.emplace_back(cycle[j], cycle[(j + 1) % len]);
      }
    }

    return test;
  };

  // small manual tests
  Add({
    3,
    {{1, 2}, {1, 3}, {3, 2}}
  });
  Add({
    7,
    {{1, 2}, {1, 3}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 7}, {7, 3}}
  });

  vector <int> counters = {10, 25, 100, 500, 1000, 3333, 10000, 33333};
  for (int cnt: counters) {
    {
      // only cycles with length 4
      auto lengths = GenerateCycleLengths(3, 3, 0, 4, 4, cnt);

      Add(GenerateChainCactus(lengths));
      Add(GenerateStarCactus(lengths));
      Add(GenerateRandomCactus(lengths));
      Add(GenerateGoodRandomCactus(lengths));
    }
  }

  // with answer "no"
  counters = {3, 15, 25, 111};
  for (int cnt: counters) {
    {
      auto lengths = GenerateCycleLengths(cnt, cnt + 4, cnt, 4, cnt + 5, cnt);

      Add(GenerateChainCactus(lengths));
      Add(GenerateStarCactus(lengths));
      Add(GenerateRandomCactus(lengths));
      Add(GenerateGoodRandomCactus(lengths));
    }
    {
      // only cycles with length cnt
      auto lengths = GenerateCycleLengths(cnt, cnt, cnt, 4, 4, 0);

      Add(GenerateRandomCactus(lengths));
    }
  }

  vector <int> odd_min = {3, 7, 13, 101, 121, 1111, 67};
  vector <int> odd_max = {3, 9, 23, 333, 121, 1111, 111};
  vector <int> odd_cnt = {2, 6, 12, 100, 4, 8, 50};
  vector <int> even_min = {4, 4, 4, 4, 5000, 4, 4};
  vector <int> even_max = {4, 8, 4, 400, 10000, 4, 500};
  vector <int> even_cnt = {1, 11, 2, 100, 5, 10000, 20};

  for (int i = 0; i < (int)odd_min.size(); i++) {
    auto lengths = GenerateCycleLengths(odd_min[i], odd_max[i], odd_cnt[i], even_min[i], even_max[i], even_cnt[i]);

    Add(GenerateChainCactus(lengths));
    Add(GenerateStarCactus(lengths));
    Add(GenerateRandomCactus(lengths));
    Add(GenerateGoodRandomCactus(lengths));
  }

  sort(tests.begin(), tests.end(), [&](const auto& x, const auto& y) {
    return make_pair(x.n, x.cactus.size()) < make_pair(y.n, y.cactus.size());
  });

  for (auto& t : tests) {
    startTest(start++);

    int m = (int)t.cactus.size();
    cout << t.n << " " << m << endl;
    for (int i = 0; i < m; i++) {
      cout << t.cactus[i].first << " " << t.cactus[i].second << endl;
    }
  }
  return 0;
}
