#include "testlib.h"
#include <iostream>
#include <vector>
#include <unordered_map>
#include <functional>
#include <algorithm>
#include <cassert>

using namespace std;

std::vector<int> read_answer(InStream& in, int n) {
    std::vector<int> answer;

    for (int i = 0; i < n; ++i) {
        int t = in.readInt(-1, n, format("a[%d]", i));

        if (t == 0) {
            in.quitf(_pe, "Incorrect output");
        }

        if (t == -1) {
            if (answer.empty()) {
                return {};
            }
            in.quitf(_pe, "Incorrect output");
        }
        answer.push_back(t - 1);
    }

    if (answer.size() != n) {
        in.quitf(_pe, "Incorrect size of output");  // Should not be reachable
    }
    return answer;
}

// Input data:
int n;
std::vector<int> op1, op2, op3, op4;
std::vector<int> operations;
int m;
std::vector<std::pair<int, int>> edges;

void read_input() {
    n = inf.readInt(1, 100'000); // todo
    inf.readEoln();
    for (int i = 0; i < n; ++i) {
        std::string op_type, op_result;

        op_type = inf.readToken("set|unset");
        inf.readSpace();
        op_result = inf.readToken("true|false");
        inf.readEoln();

        if (op_type == "set" && op_result == "true") {
            op1.push_back(i);
            operations.push_back(1);

        } else if (op_type == "unset" && op_result == "true") {
            op2.push_back(i);
            operations.push_back(2);

        } else if (op_type == "set" && op_result == "false") {
            op3.push_back(i);
            operations.push_back(3);

        } else if (op_type == "unset" && op_result == "false") {
            op4.push_back(i);
            operations.push_back(4);
        }
    }

    m = inf.readInt(0, 100'000);
    inf.readEoln();

    for (int i = 0; i < m; ++i) {
        int a, b;
        a = inf.readInt(1, n);
        inf.readSpace();
        b = inf.readInt(1, n);
        inf.readEoln();
        edges.emplace_back(a - 1, b - 1);
    }
    inf.readEof();
}

std::unordered_map<int, std::vector<int>> build_dag() {
    std::unordered_map<int, std::vector<int>> dag;

    for (auto [a, b]: edges) {
        dag[a].push_back(b);

        // (1, 4) || (3, 4) => (2, 4)
        if ((operations[a] == 1 || operations[a] == 3) && operations[b] == 4) {
            if (op2.empty()) {
                // No solution!
                dag[a].push_back(a);
                return dag;
            }
            dag[op2[0]].push_back(b);
        }

        // (4, 2) || (4, 3) => (4, 1)
        if (operations[a] == 4 && (operations[b] == 2 || operations[b] == 3)) {
            if (op1.empty()) {
                // No solution!
                dag[a].push_back(a);
                return dag;
            }
            dag[a].push_back(op1[0]);
        }
    }

    for (int i: op1) {
        for (int j: op3) {
            dag[i].push_back(j);
        }
    }
    for (int i: op3) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    for (int i: op1) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    return dag;
}

std::optional<std::vector<int>> top_sort(std::unordered_map<int, std::vector<int>> &dag) {
    std::vector<int> result;
    std::vector<bool> gray(n, false);
    std::vector<bool> black(n, false);
    std::function<bool(int)> dfs;

    dfs = [&](int v){
        if (black[v]) {
            return false;
        }
        if (gray[v]) {
            return false;
        }
        gray[v] = true;
        for (auto u: dag[v]) {
            if (dfs(u)) {
                return true;
            }
        }

        result.push_back(v);
        black[v] = true;
        return false;
    };

    for (int i = 0; i < n; ++i) {
        if (dfs(i)) {
            std::cout << -1 << std::endl;
            return std::nullopt;
        }
    }

    std::reverse(result.begin(), result.end());
    return result;
}

struct StateMachine {
    StateMachine() : state(false) {}

    bool go(int operation_type) {
        if (operation_type == 1) {
            if (state) {
                return false;
            }
            state = true;
            return true;
        }

        if (operation_type == 2) {
            if (!state) {
                return false;
            }
            state = false;
            return true;
        }

        if (operation_type == 3) {
            if (!state) {
                return false;
            }
            return true;
        }

        if (operation_type == 4) {
            if (state) {
                return false;
            }
            return true;
        }

        assert(false);
    }

private:
    bool state;
};

void check_answer(
        std::unordered_map<int, std::vector<int>> &dag,
        std::vector<int> &participant,
        std::vector<int> &jury
) {
    std::vector<int> deg_in(n);
    for (auto const &[v, vertexes]: dag) {
        for (int u: vertexes) {
            ++deg_in[u];
        }
    }
    if (participant.empty() && !jury.empty()) {
        quitf(_wa, "Jury has the answer but participant has not");
    }
    if (!participant.empty() && jury.empty()) {
        quitf(_wa, "Jury has the answer but participant has not");
    }
    if (participant.empty() && jury.empty()) {
        return;
    }

    // 1. Check transition
    StateMachine st;
    for (int v: participant) {
        if (!st.go(operations[v])) {
            quitf(_wa, "Incorrect transition");
        }
    }

    // 2. Check DAG
    for (int v: participant) {
        if (deg_in[v] != 0) {
            quitf(_wa, "Top sort is incorrect");
        }

        for (int u: dag[v]) {
            --deg_in[u];
            if (deg_in[u] < 0) {
                quitf(_wa, "Top sort is incorrect");
            }
        }
    }

    // Check that the participant answer is a permutation of a [1...n]
    std::vector<int> count(n);
    for (int op: participant) {
        if (op < 0 || op >= n) {
            quitf(_wa, "Incorrect operation number");
        }

        ++count[op];

        if (count[op] != 1) {
            quitf(_wa, "Operation %d appeared twice", op);
        }
    }

    // Check again (redundant)
    // todo: delete it
    std::sort(participant.begin(), participant.end());
    std::sort(jury.begin(), jury.end());
    for (int i = 0; i < participant.size(); ++i) {
        if (participant[i] != jury[i]) {
            quitf(_fail, "Participant answer is not a permutation of a jury answer");
        }
    }
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);

    read_input();
    auto dag = build_dag();

    std::vector<int> participant = read_answer(ouf, n);
    std::vector<int> jury = read_answer(ans, n);

    check_answer(dag, participant, jury);

    quitf(_ok, "OK");
    return 0;
}
