#include "testlib.h"
#include <unordered_map>
#include <unordered_set>
#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <string>
#include <tuple>
#include <map>
#include <numeric>

using namespace std;

typedef std::unordered_map<std::string, std::string> PARAMS;

PARAMS parse_params(int argc, char *argv[]) {
    PARAMS params;

    for (int i = 1; i < argc; i++) {
        std::string s = std::string(argv[i]);
        if (s.find('=') != std::string::npos) {
            auto pos = s.find('=');
            params[s.substr(0, pos)] = s.substr(pos + 1);
        }
    }

    return params;
}

std::unordered_map<int, std::vector<int>> build_dag(
        std::vector<int> const &op1,
        std::vector<int> const &op2,
        std::vector<int> const &op3
) {
    std::unordered_map<int, std::vector<int>> dag;

    for (int i: op1) {
        for (int j: op3) {
            dag[i].push_back(j);
        }
    }
    for (int i: op3) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    for (int i: op1) {
        for (int j: op2) {
            dag[i].push_back(j);
        }
    }
    return dag;
}

std::optional<std::vector<int>> top_sort(
        int n,
        std::unordered_map<int, std::vector<int>> &dag,
        std::vector<int> const &op1
) {
    std::vector<int> result;
    std::vector<bool> gray(n, false);
    std::vector<bool> black(n, false);
    std::function<bool(int)> dfs;

    dfs = [&](int v){
        if (black[v]) {
            return false;
        }
        if (gray[v]) {
            return true;
        }
        gray[v] = true;
        for (auto u: dag[v]) {
            if (dfs(u)) {
                return true;
            }
        }

        result.push_back(v);
        black[v] = true;
        return false;
    };

    // todo: figure out it better, add to solution and other files, if needed
    // 1-first!
    for (int v: op1) {
        if (dfs(v)) {
            return std::nullopt;
        }
    }
    for (int i = 0; i < n; ++i) {
        if (dfs(i)) {
            return std::nullopt;
        }
    }

    std::reverse(result.begin(), result.end());
    return result;
}

int main(int argc, char **argv) {
    registerGen(argc, argv, 1);

//    int cnt_operation1 = rnd.next(2);
//    int cnt_operation2 = rnd.next(max(1, cnt_operation1 + 1));
//    int cnt_operation3 = rnd.next(max(1, cnt_operation1 + 1));

    PARAMS params = parse_params(argc, argv);
    int n1 = std::stoi(params["n1"]);
    int n2 = std::stoi(params["n2"]);
    int n3 = std::stoi(params["n3"]);
    int n4 = std::stoi(params["n4"]);
    int m = std::stoi(params["m"]);
    bool guarantee_answer_exists = false;
    if (params.count("ans_exists")) {
        guarantee_answer_exists = true;
    }

    int n = n1 + n2 + n3 + n4;
    std::vector<int> op1, op2, op3, op4;
    std::vector<int> operations(n);

    std::vector<int> p = rnd.perm(n);

//    for (int x: p) std::cout << x << " "; std::cout << std::endl;

    int op_type = 1;

    for (int i = 0; i < n; ++i) {
        if (op_type == 1 && n1 == op1.size()) op_type = 2;
        if (op_type == 2 && n2 == op2.size()) op_type = 3;
        if (op_type == 3 && n3 == op3.size()) op_type = 4;

        operations[p[i]] = op_type;
        if (op_type == 1) op1.push_back(p[i]);
        if (op_type == 2) op2.push_back(p[i]);
        if (op_type == 3) op3.push_back(p[i]);
        if (op_type == 4) op4.push_back(p[i]);
    }
    assert(n1 == op1.size() && n2 == op2.size() && n3 == op3.size() && n4 == op4.size());

    // Build old one first
    std::unordered_map<int, std::vector<int>> dag = build_dag(op1, op2, op3);
    std::vector<int> top_sort_order = *top_sort(n, dag, op1);

//    std::cout << "---\n";
//    for (int x: top_sort_order) std::cout << (x + 1) << ' '; std::cout << std::endl;
//    std::cout << "---\n";

    std::vector<int> pp(n);
    if (guarantee_answer_exists) {
        std::iota(pp.begin(), pp.end(), 0);
    } else {
        pp = rnd.perm(n);
    }

    // Generate edges
    std::vector<std::pair<int, int>> edges;
    for (int i = 0; i < m; ++i) {
        int a = rnd.next(n);
        int b = rnd.next(n);
        while (a == b) {
            a = rnd.next(n);
        }
        if (pp[a] > pp[b]) {
            std::swap(a, b);
        }

        edges.emplace_back(top_sort_order[a], top_sort_order[b]);
    }

    // Output
    std::cout << n << std::endl;
    for (int op: operations) {
        if (op == 1) std::cout << "set true" << std::endl;
        if (op == 2) std::cout << "unset true" << std::endl;
        if (op == 3) std::cout << "set false" << std::endl;
        if (op == 4) std::cout << "unset false" << std::endl;
    }
    std::cout << m << std::endl;
    for (auto [a, b]: edges) {
        std::cout << (a + 1) << " " << (b + 1) << std::endl;
    }
    return 0;
}
