#include <iostream>
#include <vector>
#include <unordered_map>
#include <functional>
#include <algorithm>
#include "testlib.h"

using namespace std;

// Input data:
int n;
std::vector<int> op1, op2, op3, op4;
std::vector<int> operations;
int m;
std::vector<std::pair<int, int>> edges;

void check_dag(std::unordered_map<int, std::vector<int>> &dag) {

    std::vector<bool> black(n, false);
    std::vector<bool> gray(n, false);

    std::function<bool(int)> dfs;

    dfs = [&](int v) {
        if (black[v]) {
            return false;
        }
        if (gray[v]) {
            return true;
        }
        gray[v] = true;
        for (int u: dag[v]) {
            if (dfs(u)) {
                return true;
            }
        }
        black[v] = true;
        return false;
    };
    int count = 0;
    for (int i = 0; i < n; ++i) {
        bool is_loop = dfs(i);
        if (is_loop) {
            count ++;
        }
    }
    ensuref(count < 1, "Loops are forbidden");
}

void read_and_check_input() {
    n = inf.readInt(1, 100'000, "n");
    inf.readEoln();
    for (int i = 0; i < n; ++i) {
        std::string op_type, op_result;

        op_type = inf.readToken("set|unset", "op_type[i]");
        inf.readSpace();
        op_result = inf.readToken("true|false", "op_result[i]");
        inf.readEoln();

        if (op_type == "set" && op_result == "true") {
            op1.push_back(i);
            operations.push_back(1);

        } else if (op_type == "unset" && op_result == "true") {
            op2.push_back(i);
            operations.push_back(2);

        } else if (op_type == "set" && op_result == "false") {
            op3.push_back(i);
            operations.push_back(3);

        } else if (op_type == "unset" && op_result == "false") {
            op4.push_back(i);
            operations.push_back(4);
        }
    }

    ensuref(op1.size() <= 1, "Number of 'set true' operations must be less or equal then 1");
    ensuref(op2.size() <= op1.size(), "Number of 'unset true' operations must be less or equal then"
                                      "number of 'set true' operations (0 or 1)");

    m = inf.readInt(0, 100'000, "m");
    inf.readEoln();

    std::unordered_map<int, std::vector<int>> dag;

    for (int i = 0; i < m; ++i) {
        int a, b;
        a = inf.readInt(1, n, "a[i]");
        inf.readSpace();
        b = inf.readInt(1, n, "b[i]");
        inf.readEoln();
        edges.emplace_back(a - 1, b - 1);

        dag[a - 1].push_back(b - 1);

        ensuref(a != b, "Self-loop are forbidden");
    }

    check_dag(dag);

    inf.readEof();
}



int main(int argc, char *argv[]) {
    registerValidation(argc, argv);
    read_and_check_input();

    return 0;
}
