pub mod solution {
    //{"name":"g_test_generator","group":"Manual","url":"","interactive":false,"timeLimit":2000,"tests":[{"input":"","output":""}],"testType":"single","input":{"type":"stdin","fileName":null,"pattern":null},"output":{"type":"stdout","fileName":null,"pattern":null},"languages":{"java":{"taskClass":"g_test_generator"}}}

    use crate::algo_lib::collections::slice_ext::indices::Indices;
    use crate::algo_lib::io::input::Input;
    use crate::algo_lib::io::output::Output;
    use crate::algo_lib::misc::random::Random;
    use crate::algo_lib::misc::test_type::TaskType;
    use crate::algo_lib::misc::test_type::TestType;
    use std::cmp::Ordering;
    use std::fs::File;
    use std::ops::{Add, Mul, Sub};

    type PreCalc = ();

    fn solve(_input: &mut Input, _out: &mut Output, _test_case: usize, _data: &mut PreCalc) {
        enum Command {
            Move(i64),
            Draw(i64),
            Rotate(usize),
        }
        struct Test {
            commands: Vec<Command>,
        }

        impl Test {
            fn write(&self, id: usize) {
                let mut file = File::create(id.to_string()).unwrap();
                // let mut turtle = File::create(format!("{}.py", id)).unwrap();
                let mut out = Output::new(&mut file);
                // let mut out_turtle = Output::new(&mut turtle);
                out.print(self.commands.len());
                out.print("\r\n");
                // out_turtle.print_line("import turtle");
                // out_turtle.print_line("t = turtle.Turtle()");
                // out_turtle.print_line("t.speed(5)");
                for command in &self.commands {
                    match command {
                        Command::Move(x) => {
                            out.print_line(format!("move {}\r", x));
                            // out_turtle.print_line("t.penup()");
                            // out_turtle.print_line(format!("t.forward({})", x * 5));
                            // out_turtle.print_line("t.pendown()");
                        }
                        Command::Draw(x) => {
                            out.print_line(format!("draw {}\r", x));
                            // out_turtle.print_line(format!("t.forward({})", x * 5));
                        }
                        Command::Rotate(x) => {
                            out.print_line(format!("rotate {}\r", x * 45));
                            // out_turtle.print_line(format!("t.right({})", x * 45));
                        }
                    }
                }
                out.flush();
                // out_turtle.flush();
            }
        }

        static mut RAND: Option<Random> = None;
        pub fn random() -> &'static mut Random {
            unsafe {
                if RAND.is_none() {
                    RAND = Some(Random::new(239));
                }
                RAND.as_mut().unwrap()
            }
        }

        fn gen_test(
            segs: usize,
            split_segs: usize,
            max_len: i64,
            ans: usize,
            min_ops: usize,
            max_ops: usize,
        ) -> Option<Test> {
            let c = random().next(2);
            let center = [c, c, c, c];
            let mut segments = Vec::new();
            for _ in 0..segs / ans {
                let mut base = [
                    random().next_bounds(-max_len, max_len) * 2,
                    random().next_bounds(-max_len, max_len) * 2,
                    random().next_bounds(-max_len, max_len) * 2,
                    random().next_bounds(-max_len, max_len) * 2,
                ];
                let mut dir = random().next(4);
                let len = random().next_bounds(1, max_len) * 2;
                for j in 0..ans {
                    if random().next(2) == 0 {
                        segments.push((base, dir, len));
                    } else {
                        base[dir % 4] += if dir < 4 { len } else { -len };
                        segments.push((base, dir ^ 4, len));
                        base[dir % 4] -= if dir < 4 { len } else { -len };
                    }
                    let mut next_base = [0; 4];
                    let sh = 8 / ans;
                    if j != ans - 1 {
                        for k in sh..4 {
                            next_base[k] = base[k - sh] - center[k - sh] + center[k];
                        }
                        for k in 0..sh {
                            next_base[k] = center[k] - (base[k + 4 - sh] - center[k + 4 - sh]);
                        }
                    }
                    base = next_base;
                    dir = (dir + sh) % 8;
                }
            }
            if segments.is_empty() {
                return None;
            }
            for (from, _, len) in &mut segments {
                *len /= 2;
                for i in from.indices() {
                    from[i] /= 2;
                }
            }
            let mut split_segments = Vec::new();
            for (mut from, dir, mut len) in segments {
                while len > 1000000000 {
                    let cur = random().next_bounds(1, 1000000000);
                    split_segments.push((from, dir, cur));
                    from[dir % 4] += if dir < 4 { cur } else { -cur };
                    len -= cur;
                }
                split_segments.push((from, dir, len));
            }
            if split_segments.len() > split_segs {
                return None;
            }
            let mut tries = 0;
            while split_segments.len() < split_segs {
                if tries > 1000000 {
                    break;
                }
                tries += 1;
                let pos = random().next(split_segments.len());
                let (mut from, dir, len) = split_segments[pos];
                if len == 1 {
                    continue;
                }
                let cur = random().next_bounds(1, len - 1);
                split_segments[pos].2 -= cur;
                from[dir % 4] += if dir < 4 { len - cur } else { cur - len };
                if random().next(2) == 0 {
                    split_segments.push((from, dir, cur));
                } else {
                    from[dir % 4] += if dir < 4 { cur } else { -cur };
                    split_segments.push((from, dir ^ 4, cur));
                }
            }
            for i in split_segments.indices() {
                let to = random().next(i + 1);
                split_segments.swap(i, to);
            }

            for i in split_segments.indices() {
                let (from, dir, len) = split_segments[i];
                for j in 0..i {
                    let (from2, dir2, len2) = split_segments[j];
                    if dir % 4 != dir2 % 4 {
                        continue;
                    }
                    let mut ok = false;
                    for k in 0..4 {
                        if k != dir % 4 && from[k] != from2[k] {
                            ok = true;
                            break;
                        }
                    }
                    if ok {
                        continue;
                    }
                    let mut a = [
                        from[dir % 4],
                        from[dir % 4] + if dir < 4 { len } else { -len },
                    ];
                    a.sort();
                    let mut b = [
                        from2[dir2 % 4],
                        from2[dir2 % 4] + if dir2 < 4 { len2 } else { -len2 },
                    ];
                    b.sort();
                    if a[1].min(b[1]) > a[0].max(b[0]) {
                        return None;
                    }
                }
            }

            let mut commands = Vec::new();
            let mut pos = [
                random().next_bounds(-max_len, max_len),
                random().next_bounds(-max_len, max_len),
                random().next_bounds(-max_len, max_len),
                random().next_bounds(-max_len, max_len),
            ];
            let mut dir = random().next(8);
            for (from, d, len) in split_segments {
                while pos != from {
                    let c = random().next(4);
                    if pos[c] != from[c] {
                        let n_dir = if pos[c] < from[c] { c } else { c + 4 };
                        if dir != n_dir {
                            commands.push(Command::Rotate((n_dir + 8 - dir) % 8));
                            dir = n_dir;
                        }
                        let mut delta = (pos[c] - from[c]).abs();
                        if delta > 1000000000 {
                            delta = random().next_bounds(1, 1000000000);
                        }
                        commands.push(Command::Move(delta));
                        pos[c] += if pos[c] < from[c] { delta } else { -delta };
                    }
                }
                if dir != d {
                    commands.push(Command::Rotate((d + 8 - dir) % 8));
                    dir = d;
                }
                commands.push(Command::Draw(len));
                pos[dir % 4] += if dir < 4 { len } else { -len };
            }
            if commands.len() > max_ops {
                return None;
            }
            while commands.len() < min_ops {
                if random().next(2) == 0 {
                    commands.push(Command::Rotate(random().next_bounds(1, 8)));
                } else {
                    commands.push(Command::Move(random().next_bounds(1, max_len)));
                }
            }
            Some(Test { commands })
        }

        let mut tests = Vec::new();

        for max_len in [20, 1000, 1000000000] {
            // for max_len in [20] {
            for segs in [10, 80, 1000] {
                // for segs in [10] {
                for split_segs in [segs, segs * 2] {
                    for ans in [1, 2, 4, 8] {
                        for _ in 0..2 {
                            let ml = random().next_bounds(1, max_len);
                            let s = random().next_bounds(1, segs);
                            let ss = random().next_bounds(s, split_segs);
                            let option = gen_test(s, ss, ml, ans, 1, 50000);
                            if option.is_some() {
                                println!("ans = {}", ans);
                            }
                            tests.push(option);
                        }
                    }
                }
            }
        }
        for ans in [1, 2, 4, 8] {
            tests.push(gen_test(2000, 2000, 1000000000, ans, 50000, 50000));
        }

        fn check(test: &Test) -> bool {
            #[derive(Clone, Copy, Eq, PartialEq)]
            struct Number {
                a: i128,
                b: i128,
            }

            impl Number {
                fn new(a: i128, b: i128) -> Self {
                    Self { a, b }
                }

                fn from(a: i128) -> Self {
                    Self { a, b: 0 }
                }
            }

            impl Add for Number {
                type Output = Self;

                fn add(self, rhs: Self) -> Self::Output {
                    Self {
                        a: self.a + rhs.a,
                        b: self.b + rhs.b,
                    }
                }
            }

            impl Sub for Number {
                type Output = Self;

                fn sub(self, rhs: Self) -> Self::Output {
                    Self {
                        a: self.a - rhs.a,
                        b: self.b - rhs.b,
                    }
                }
            }

            impl Mul for Number {
                type Output = Self;

                fn mul(self, rhs: Self) -> Self::Output {
                    Self {
                        a: self.a * rhs.a + 2 * self.b * rhs.b,
                        b: self.a * rhs.b + self.b * rhs.a,
                    }
                }
            }

            impl Ord for Number {
                fn cmp(&self, other: &Self) -> Ordering {
                    if *self == *other {
                        return Ordering::Equal;
                    }
                    let qb = other.b - self.b;
                    let qa = other.a - self.a;
                    if qb >= 0 && qa >= 0 {
                        return Ordering::Less;
                    }
                    if qa <= 0 && qb <= 0 {
                        return Ordering::Greater;
                    }
                    if qb * qb * 2 < qa * qa {
                        if qa < 0 {
                            Ordering::Greater
                        } else {
                            Ordering::Less
                        }
                    } else {
                        if qa < 0 {
                            Ordering::Less
                        } else {
                            Ordering::Greater
                        }
                    }
                }
            }

            impl PartialOrd for Number {
                fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
                    Some(self.cmp(other))
                }
            }

            #[derive(Clone, Copy, Ord, PartialOrd, Eq, PartialEq)]
            struct Point {
                x: Number,
                y: Number,
            }

            impl Mul<Point> for Number {
                type Output = Point;

                fn mul(self, rhs: Point) -> Self::Output {
                    Point {
                        x: self * rhs.x,
                        y: self * rhs.y,
                    }
                }
            }

            impl Add for Point {
                type Output = Self;

                fn add(self, rhs: Self) -> Self::Output {
                    Self {
                        x: self.x + rhs.x,
                        y: self.y + rhs.y,
                    }
                }
            }

            fn apply(d: usize, mut p: Point) -> Point {
                for _ in 0..d {
                    let mut q = Point {
                        x: Number::new(0, 1) * p.x - Number::new(0, 1) * p.y,
                        y: Number::new(0, 1) * p.x + Number::new(0, 1) * p.y,
                    };
                    assert!(q.x.a % 2 == 0);
                    q.x.a /= 2;
                    assert!(q.y.a % 2 == 0);
                    q.y.a /= 2;
                    assert!(q.x.b % 2 == 0);
                    q.x.b /= 2;
                    assert!(q.y.b % 2 == 0);
                    q.y.b /= 2;
                    p = q;
                }
                p
            }

            fn collinear(p: Point, q: Point, r: Point) -> bool {
                let a = (r.x - q.x) * (q.y - p.y);
                let b = (r.y - q.y) * (q.x - p.x);
                a == b
            }

            fn is_between(mut l: Number, mut r: Number, x: Number) -> bool {
                if l > r {
                    std::mem::swap(&mut l, &mut r);
                }
                l < x && x < r
            }

            let mut segs = Vec::new();
            let mut pos = Point {
                x: Number::from(0),
                y: Number::from(0),
            };
            let mut dir = Point {
                x: Number::from(4),
                y: Number::from(0),
            };
            for command in &test.commands {
                match command {
                    Command::Move(l) => {
                        pos = pos + Number::from(*l as i128) * dir;
                    }
                    Command::Draw(l) => {
                        let next = pos + Number::from(*l as i128) * dir;
                        segs.push((pos, next));
                        pos = next;
                    }
                    Command::Rotate(d) => {
                        dir = apply(*d, dir);
                    }
                }
            }
            for i in segs.indices() {
                let (li, ri) = segs[i];
                for j in 0..i {
                    let (lj, rj) = segs[j];
                    if collinear(li, ri, lj) && collinear(li, ri, rj) {
                        if is_between(li.x, ri.x, lj.x) {
                            return false;
                        }
                        if is_between(li.x, ri.x, rj.x) {
                            return false;
                        }
                        if is_between(li.y, ri.y, lj.y) {
                            return false;
                        }
                        if is_between(li.y, ri.y, rj.y) {
                            return false;
                        }
                    }
                }
            }
            true
        }

        let mut tests = tests.into_iter().filter_map(|x| x).collect::<Vec<_>>();
        // tests.sort_by_key(|x| x.commands.len());
        let mut id = 6;
        for test in tests {
            if !check(&test) {
                continue;
            }
            test.write(id);
            id += 1;
        }
    }

    pub static TEST_TYPE: TestType = TestType::Single;
    pub static TASK_TYPE: TaskType = TaskType::Classic;

    pub(crate) fn run(mut input: Input, mut output: Output) -> bool {
        let mut pre_calc = ();

        match TEST_TYPE {
            TestType::Single => solve(&mut input, &mut output, 1, &mut pre_calc),
            TestType::MultiNumber => {
                let t = input.read();
                for i in 1..=t {
                    solve(&mut input, &mut output, i, &mut pre_calc);
                }
            }
            TestType::MultiEof => {
                let mut i = 1;
                while input.peek().is_some() {
                    solve(&mut input, &mut output, i, &mut pre_calc);
                    i += 1;
                }
            }
        }
        output.flush();
        // match TASK_TYPE {
        //     TaskType::Classic => input.is_empty(),
        //     TaskType::Interactive => true,
        // }
        true
    }
}
pub mod algo_lib {
    #![allow(clippy::too_many_arguments)]
    #![allow(clippy::type_complexity)]
    #![allow(clippy::missing_safety_doc)]
    #![allow(clippy::legacy_numeric_constants)]

    pub mod collections {
        pub mod slice_ext {
            pub mod indices {
                use std::ops::Range;

                pub trait Indices {
                    fn indices(&self) -> Range<usize>;
                }

                impl<T> Indices for [T] {
                    fn indices(&self) -> Range<usize> {
                        0..self.len()
                    }
                }
            }
        }
        pub mod vec_ext {
            pub mod default {
                pub fn default_vec<T: Default>(len: usize) -> Vec<T> {
                    let mut v = Vec::with_capacity(len);
                    for _ in 0..len {
                        v.push(T::default());
                    }
                    v
                }
            }
        }
    }
    pub mod io {
        pub mod input {
            use crate::algo_lib::collections::vec_ext::default::default_vec;
            use std::io::Read;
            use std::mem::MaybeUninit;

            pub struct Input<'s> {
                input: &'s mut (dyn Read + Send),
                buf: Vec<u8>,
                at: usize,
                buf_read: usize,
            }

            macro_rules! read_impl {
                ($t: ty, $read_name: ident, $read_vec_name: ident) => {
                    pub fn $read_name(&mut self) -> $t {
                        self.read()
                    }

                    pub fn $read_vec_name(&mut self, len: usize) -> Vec<$t> {
                        self.read_vec(len)
                    }
                };

                ($t: ty, $read_name: ident, $read_vec_name: ident, $read_pair_vec_name: ident) => {
                    read_impl!($t, $read_name, $read_vec_name);

                    pub fn $read_pair_vec_name(&mut self, len: usize) -> Vec<($t, $t)> {
                        self.read_vec(len)
                    }
                };
            }

            impl<'s> Input<'s> {
                const DEFAULT_BUF_SIZE: usize = 4096;

                pub fn new(input: &'s mut (dyn Read + Send)) -> Self {
                    Self {
                        input,
                        buf: default_vec(Self::DEFAULT_BUF_SIZE),
                        at: 0,
                        buf_read: 0,
                    }
                }

                pub fn new_with_size(input: &'s mut (dyn Read + Send), buf_size: usize) -> Self {
                    Self {
                        input,
                        buf: default_vec(buf_size),
                        at: 0,
                        buf_read: 0,
                    }
                }

                pub fn get(&mut self) -> Option<u8> {
                    if self.refill_buffer() {
                        let res = self.buf[self.at];
                        self.at += 1;
                        if res == b'\r' {
                            if self.refill_buffer() && self.buf[self.at] == b'\n' {
                                self.at += 1;
                            }
                            return Some(b'\n');
                        }
                        Some(res)
                    } else {
                        None
                    }
                }

                pub fn peek(&mut self) -> Option<u8> {
                    if self.refill_buffer() {
                        let res = self.buf[self.at];
                        Some(if res == b'\r' { b'\n' } else { res })
                    } else {
                        None
                    }
                }

                pub fn skip_whitespace(&mut self) {
                    while let Some(b) = self.peek() {
                        if !b.is_ascii_whitespace() {
                            return;
                        }
                        self.get();
                    }
                }

                pub fn next_token(&mut self) -> Option<Vec<u8>> {
                    self.skip_whitespace();
                    let mut res = Vec::new();
                    while let Some(c) = self.get() {
                        if c.is_ascii_whitespace() {
                            break;
                        }
                        res.push(c);
                    }
                    if res.is_empty() {
                        None
                    } else {
                        Some(res)
                    }
                }

                //noinspection RsSelfConvention
                pub fn is_exhausted(&mut self) -> bool {
                    self.peek().is_none()
                }

                //noinspection RsSelfConvention
                pub fn is_empty(&mut self) -> bool {
                    self.skip_whitespace();
                    self.is_exhausted()
                }

                pub fn read<T: Readable>(&mut self) -> T {
                    T::read(self)
                }

                pub fn read_vec<T: Readable>(&mut self, size: usize) -> Vec<T> {
                    let mut res = Vec::with_capacity(size);
                    for _ in 0..size {
                        res.push(self.read());
                    }
                    res
                }

                pub fn read_char(&mut self) -> u8 {
                    self.skip_whitespace();
                    self.get().unwrap()
                }

                read_impl!(u32, read_unsigned, read_unsigned_vec);
                read_impl!(u64, read_u64, read_u64_vec);
                read_impl!(usize, read_size, read_size_vec, read_size_pair_vec);
                read_impl!(i32, read_int, read_int_vec, read_int_pair_vec);
                read_impl!(i64, read_long, read_long_vec, read_long_pair_vec);
                read_impl!(i128, read_i128, read_i128_vec);

                fn refill_buffer(&mut self) -> bool {
                    if self.at == self.buf_read {
                        self.at = 0;
                        self.buf_read = self.input.read(&mut self.buf).unwrap();
                        self.buf_read != 0
                    } else {
                        true
                    }
                }
            }

            pub trait Readable {
                fn read(input: &mut Input) -> Self;
            }

            impl Readable for u8 {
                fn read(input: &mut Input) -> Self {
                    input.read_char()
                }
            }

            impl<T: Readable> Readable for Vec<T> {
                fn read(input: &mut Input) -> Self {
                    let size = input.read();
                    input.read_vec(size)
                }
            }

            impl<T: Readable, const SIZE: usize> Readable for [T; SIZE] {
                fn read(input: &mut Input) -> Self {
                    unsafe {
                        let mut res = MaybeUninit::<[T; SIZE]>::uninit();
                        for i in 0..SIZE {
                            let ptr: *mut T = (*res.as_mut_ptr()).as_mut_ptr();
                            ptr.add(i).write(input.read::<T>());
                        }
                        res.assume_init()
                    }
                }
            }

            macro_rules! read_integer {
    ($($t:ident)+) => {$(
        impl Readable for $t {
            fn read(input: &mut Input) -> Self {
                input.skip_whitespace();
                let mut c = input.get().unwrap();
                let sgn = match c {
                    b'-' => {
                        c = input.get().unwrap();
                        true
                    }
                    b'+' => {
                        c = input.get().unwrap();
                        false
                    }
                    _ => false,
                };
                let mut res = 0;
                loop {
                    assert!(c.is_ascii_digit());
                    res *= 10;
                    let d = (c - b'0') as $t;
                    if sgn {
                        res -= d;
                    } else {
                        res += d;
                    }
                    match input.get() {
                        None => break,
                        Some(ch) => {
                            if ch.is_ascii_whitespace() {
                                break;
                            } else {
                                c = ch;
                            }
                        }
                    }
                }
                res
            }
        }
    )+};
}

            read_integer!(i8 i16 i32 i64 i128 isize u16 u32 u64 u128 usize);

            macro_rules! tuple_readable {
    ($($name:ident)+) => {
        impl<$($name: Readable), +> Readable for ($($name,)+) {
            fn read(input: &mut Input) -> Self {
                ($($name::read(input),)+)
            }
        }
    }
}

            tuple_readable! {T}
            tuple_readable! {T U}
            tuple_readable! {T U V}
            tuple_readable! {T U V X}
            tuple_readable! {T U V X Y}
            tuple_readable! {T U V X Y Z}
            tuple_readable! {T U V X Y Z A}
            tuple_readable! {T U V X Y Z A B}
            tuple_readable! {T U V X Y Z A B C}
            tuple_readable! {T U V X Y Z A B C D}
            tuple_readable! {T U V X Y Z A B C D E}
            tuple_readable! {T U V X Y Z A B C D E F}

            impl Read for Input<'_> {
                fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
                    if self.at == self.buf_read {
                        self.input.read(buf)
                    } else {
                        let mut i = 0;
                        while i < buf.len() && self.at < self.buf_read {
                            buf[i] = self.buf[self.at];
                            i += 1;
                            self.at += 1;
                        }
                        Ok(i)
                    }
                }
            }
        }
        pub mod output {
            use crate::algo_lib::collections::vec_ext::default::default_vec;
            use std::cmp::Reverse;
            use std::io::stderr;
            use std::io::Stderr;
            use std::io::Write;

            #[derive(Copy, Clone)]
            pub enum BoolOutput {
                YesNo,
                YesNoCaps,
                PossibleImpossible,
                Custom(&'static str, &'static str),
            }

            impl BoolOutput {
                pub fn output(&self, output: &mut Output, val: bool) {
                    (if val { self.yes() } else { self.no() }).write(output);
                }

                fn yes(&self) -> &str {
                    match self {
                        BoolOutput::YesNo => "Yes",
                        BoolOutput::YesNoCaps => "YES",
                        BoolOutput::PossibleImpossible => "Possible",
                        BoolOutput::Custom(yes, _) => yes,
                    }
                }

                fn no(&self) -> &str {
                    match self {
                        BoolOutput::YesNo => "No",
                        BoolOutput::YesNoCaps => "NO",
                        BoolOutput::PossibleImpossible => "Impossible",
                        BoolOutput::Custom(_, no) => no,
                    }
                }
            }

            pub struct Output<'s> {
                output: &'s mut dyn Write,
                buf: Vec<u8>,
                at: usize,
                auto_flush: bool,
                bool_output: BoolOutput,
                precision: Option<usize>,
            }

            impl<'s> Output<'s> {
                const DEFAULT_BUF_SIZE: usize = 4096;

                pub fn new(output: &'s mut dyn Write) -> Self {
                    Self {
                        output,
                        buf: default_vec(Self::DEFAULT_BUF_SIZE),
                        at: 0,
                        auto_flush: false,
                        bool_output: BoolOutput::YesNoCaps,
                        precision: None,
                    }
                }

                pub fn new_with_auto_flush(output: &'s mut dyn Write) -> Self {
                    Self {
                        output,
                        buf: default_vec(Self::DEFAULT_BUF_SIZE),
                        at: 0,
                        auto_flush: true,
                        bool_output: BoolOutput::YesNoCaps,
                        precision: None,
                    }
                }

                pub fn flush(&mut self) {
                    if self.at != 0 {
                        self.output.write_all(&self.buf[..self.at]).unwrap();
                        self.output.flush().unwrap();
                        self.at = 0;
                    }
                }

                pub fn print<T: Writable>(&mut self, s: T) {
                    s.write(self);
                    self.maybe_flush();
                }

                pub fn print_line<T: Writable>(&mut self, s: T) {
                    self.print(s);
                    self.put(b'\n');
                    self.maybe_flush();
                }

                pub fn put(&mut self, b: u8) {
                    self.buf[self.at] = b;
                    self.at += 1;
                    if self.at == self.buf.len() {
                        self.flush();
                    }
                }

                pub fn maybe_flush(&mut self) {
                    if self.auto_flush {
                        self.flush();
                    }
                }

                pub fn print_per_line<T: Writable>(&mut self, arg: &[T]) {
                    self.print_per_line_iter(arg.iter());
                }

                pub fn print_iter<T: Writable, I: Iterator<Item = T>>(&mut self, iter: I) {
                    let mut first = true;
                    for e in iter {
                        if first {
                            first = false;
                        } else {
                            self.put(b' ');
                        }
                        e.write(self);
                    }
                }

                pub fn print_line_iter<T: Writable, I: Iterator<Item = T>>(&mut self, iter: I) {
                    self.print_iter(iter);
                    self.put(b'\n');
                }

                pub fn print_per_line_iter<T: Writable, I: Iterator<Item = T>>(&mut self, iter: I) {
                    for e in iter {
                        e.write(self);
                        self.put(b'\n');
                    }
                }

                pub fn set_bool_output(&mut self, bool_output: BoolOutput) {
                    self.bool_output = bool_output;
                }
                pub fn set_precision(&mut self, precision: usize) {
                    self.precision = Some(precision);
                }
                pub fn reset_precision(&mut self) {
                    self.precision = None;
                }
                pub fn get_precision(&self) -> Option<usize> {
                    self.precision
                }
            }

            impl Write for Output<'_> {
                fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
                    let mut start = 0usize;
                    let mut rem = buf.len();
                    while rem > 0 {
                        let len = (self.buf.len() - self.at).min(rem);
                        self.buf[self.at..self.at + len].copy_from_slice(&buf[start..start + len]);
                        self.at += len;
                        if self.at == self.buf.len() {
                            self.flush();
                        }
                        start += len;
                        rem -= len;
                    }
                    self.maybe_flush();
                    Ok(buf.len())
                }

                fn flush(&mut self) -> std::io::Result<()> {
                    self.flush();
                    Ok(())
                }
            }

            pub trait Writable {
                fn write(&self, output: &mut Output);
            }

            impl Writable for &str {
                fn write(&self, output: &mut Output) {
                    output.write_all(self.as_bytes()).unwrap();
                }
            }

            impl Writable for String {
                fn write(&self, output: &mut Output) {
                    output.write_all(self.as_bytes()).unwrap();
                }
            }

            impl Writable for char {
                fn write(&self, output: &mut Output) {
                    output.put(*self as u8);
                }
            }

            impl Writable for u8 {
                fn write(&self, output: &mut Output) {
                    output.put(*self);
                }
            }

            impl<T: Writable> Writable for [T] {
                fn write(&self, output: &mut Output) {
                    output.print_iter(self.iter());
                }
            }

            impl<T: Writable, const N: usize> Writable for [T; N] {
                fn write(&self, output: &mut Output) {
                    output.print_iter(self.iter());
                }
            }

            impl<T: Writable + ?Sized> Writable for &T {
                fn write(&self, output: &mut Output) {
                    T::write(self, output)
                }
            }

            impl<T: Writable> Writable for Vec<T> {
                fn write(&self, output: &mut Output) {
                    self.as_slice().write(output);
                }
            }

            impl Writable for () {
                fn write(&self, _output: &mut Output) {}
            }

            macro_rules! write_to_string {
    ($($t:ident)+) => {$(
        impl Writable for $t {
            fn write(&self, output: &mut Output) {
                self.to_string().write(output);
            }
        }
    )+};
}

            write_to_string!(u16 u32 u64 u128 usize i8 i16 i32 i64 i128 isize);

            macro_rules! tuple_writable {
    ($name0:ident $($name:ident: $id:tt )*) => {
        impl<$name0: Writable, $($name: Writable,)*> Writable for ($name0, $($name,)*) {
            fn write(&self, out: &mut Output) {
                self.0.write(out);
                $(
                out.put(b' ');
                self.$id.write(out);
                )*
            }
        }
    }
}

            tuple_writable! {T}
            tuple_writable! {T U:1}
            tuple_writable! {T U:1 V:2}
            tuple_writable! {T U:1 V:2 X:3}
            tuple_writable! {T U:1 V:2 X:3 Y:4}
            tuple_writable! {T U:1 V:2 X:3 Y:4 Z:5}
            tuple_writable! {T U:1 V:2 X:3 Y:4 Z:5 A:6}
            tuple_writable! {T U:1 V:2 X:3 Y:4 Z:5 A:6 B:7}
            tuple_writable! {T U:1 V:2 X:3 Y:4 Z:5 A:6 B:7 C:8}

            impl<T: Writable> Writable for Option<T> {
                fn write(&self, output: &mut Output) {
                    match self {
                        None => (-1).write(output),
                        Some(t) => t.write(output),
                    }
                }
            }

            impl Writable for bool {
                fn write(&self, output: &mut Output) {
                    let bool_output = output.bool_output;
                    bool_output.output(output, *self)
                }
            }

            impl<T: Writable> Writable for Reverse<T> {
                fn write(&self, output: &mut Output) {
                    self.0.write(output);
                }
            }

            static mut ERR: Option<Stderr> = None;

            pub fn err() -> Output<'static> {
                unsafe {
                    if ERR.is_none() {
                        ERR = Some(stderr());
                    }
                    Output::new_with_auto_flush(ERR.as_mut().unwrap())
                }
            }
        }
    }
    pub mod misc {
        pub mod random {
            use crate::algo_lib::collections::slice_ext::indices::Indices;
            use crate::algo_lib::numbers::num_traits::algebra::IntegerSemiRingWithSub;
            use crate::algo_lib::numbers::num_traits::primitive::Primitive;
            use std::ops::Rem;
            use std::time::SystemTime;

            const NN: usize = 312;
            const MM: usize = 156;
            const MATRIX_A: u64 = 0xB5026F5AA96619E9;
            const UM: u64 = 0xFFFFFFFF80000000;
            const LM: u64 = 0x7FFFFFFF;
            const F: u64 = 6364136223846793005;
            const MAG01: [u64; 2] = [0, MATRIX_A];

            pub struct Random {
                mt: [u64; NN],
                index: usize,
            }

            impl Random {
                pub fn new(seed: u64) -> Self {
                    let mut res = Self {
                        mt: [0u64; NN],
                        index: NN,
                    };
                    res.mt[0] = seed;
                    for i in 1..NN {
                        res.mt[i] = F
                            .wrapping_mul(res.mt[i - 1] ^ (res.mt[i - 1] >> 62))
                            .wrapping_add(i as u64);
                    }
                    res
                }

                pub fn gen(&mut self) -> u64 {
                    if self.index == NN {
                        for i in 0..(NN - MM) {
                            let x = (self.mt[i] & UM) | (self.mt[i + 1] & LM);
                            self.mt[i] = self.mt[i + MM] ^ (x >> 1) ^ MAG01[(x & 1) as usize];
                        }
                        for i in (NN - MM)..(NN - 1) {
                            let x = (self.mt[i] & UM) | (self.mt[i + 1] & LM);
                            self.mt[i] = self.mt[i + MM - NN] ^ (x >> 1) ^ MAG01[(x & 1) as usize];
                        }
                        let x = (self.mt[NN - 1] & UM) | (self.mt[0] & LM);
                        self.mt[NN - 1] = self.mt[MM - 1] ^ (x >> 1) ^ MAG01[(x & 1) as usize];
                        self.index = 0;
                    }
                    let mut x = self.mt[self.index];
                    self.index += 1;
                    x ^= (x >> 29) & 0x5555555555555555;
                    x ^= (x << 17) & 0x71D67FFFEDA60000;
                    x ^= (x << 37) & 0xFFF7EEE000000000;
                    x ^= x >> 43;
                    x
                }

                pub fn next<T: Rem<Output = T> + Primitive<u64>>(&mut self, n: T) -> T
                where
                    u64: Primitive<T>,
                {
                    (self.gen() % n.to()).to()
                }

                pub fn next_bounds<T: IntegerSemiRingWithSub + Primitive<u64>>(
                    &mut self,
                    f: T,
                    t: T,
                ) -> T
                where
                    u64: Primitive<T>,
                {
                    f + self.next(t - f + T::one())
                }
            }

            static mut RAND: Option<Random> = None;

            pub fn random() -> &'static mut Random {
                unsafe {
                    if RAND.is_none() {
                        RAND = Some(Random::new(
                            (SystemTime::UNIX_EPOCH.elapsed().unwrap().as_nanos()
                                & 0xFFFFFFFFFFFFFFFF) as u64,
                        ));
                    }
                    RAND.as_mut().unwrap()
                }
            }

            pub trait Shuffle {
                fn shuffle(&mut self);
            }

            impl<T> Shuffle for [T] {
                fn shuffle(&mut self) {
                    for i in self.indices() {
                        let at = random().next(i + 1);
                        self.swap(i, at);
                    }
                }
            }
        }
        pub mod test_type {
            pub enum TestType {
                Single,
                MultiNumber,
                MultiEof,
            }

            pub enum TaskType {
                Classic,
                Interactive,
            }
        }
    }
    pub mod numbers {
        pub mod num_traits {
            pub mod algebra {
                use crate::algo_lib::numbers::num_traits::invertible::Invertible;
                use std::ops::Add;
                use std::ops::AddAssign;
                use std::ops::Div;
                use std::ops::DivAssign;
                use std::ops::Mul;
                use std::ops::MulAssign;
                use std::ops::Neg;
                use std::ops::Rem;
                use std::ops::RemAssign;
                use std::ops::Sub;
                use std::ops::SubAssign;

                pub trait Zero {
                    fn zero() -> Self;
                }

                pub trait One {
                    fn one() -> Self;
                }

                pub trait AdditionMonoid:
                    Add<Output = Self> + AddAssign + Zero + Eq + Sized
                {
                }

                impl<T: Add<Output = Self> + AddAssign + Zero + Eq> AdditionMonoid for T {}

                pub trait AdditionMonoidWithSub:
                    AdditionMonoid + Sub<Output = Self> + SubAssign
                {
                }

                impl<T: AdditionMonoid + Sub<Output = Self> + SubAssign> AdditionMonoidWithSub for T {}

                pub trait AdditionGroup: AdditionMonoidWithSub + Neg<Output = Self> {}

                impl<T: AdditionMonoidWithSub + Neg<Output = Self>> AdditionGroup for T {}

                pub trait MultiplicationMonoid:
                    Mul<Output = Self> + MulAssign + One + Eq + Sized
                {
                }

                impl<T: Mul<Output = Self> + MulAssign + One + Eq> MultiplicationMonoid for T {}

                pub trait IntegerMultiplicationMonoid:
                    MultiplicationMonoid
                    + Div<Output = Self>
                    + Rem<Output = Self>
                    + DivAssign
                    + RemAssign
                {
                }

                impl<
                        T: MultiplicationMonoid
                            + Div<Output = Self>
                            + Rem<Output = Self>
                            + DivAssign
                            + RemAssign,
                    > IntegerMultiplicationMonoid for T
                {
                }

                pub trait MultiplicationGroup:
                    MultiplicationMonoid
                    + Div<Output = Self>
                    + DivAssign
                    + Invertible<Output = Self>
                {
                }

                impl<
                        T: MultiplicationMonoid
                            + Div<Output = Self>
                            + DivAssign
                            + Invertible<Output = Self>,
                    > MultiplicationGroup for T
                {
                }

                pub trait SemiRing: AdditionMonoid + MultiplicationMonoid {}

                impl<T: AdditionMonoid + MultiplicationMonoid> SemiRing for T {}

                pub trait SemiRingWithSub: AdditionMonoidWithSub + SemiRing {}

                impl<T: AdditionMonoidWithSub + SemiRing> SemiRingWithSub for T {}

                pub trait Ring: SemiRing + AdditionGroup {}

                impl<T: SemiRing + AdditionGroup> Ring for T {}

                pub trait IntegerSemiRing: SemiRing + IntegerMultiplicationMonoid {}

                impl<T: SemiRing + IntegerMultiplicationMonoid> IntegerSemiRing for T {}

                pub trait IntegerSemiRingWithSub: SemiRingWithSub + IntegerSemiRing {}

                impl<T: SemiRingWithSub + IntegerSemiRing> IntegerSemiRingWithSub for T {}

                pub trait IntegerRing: IntegerSemiRing + Ring {}

                impl<T: IntegerSemiRing + Ring> IntegerRing for T {}

                pub trait Field: Ring + MultiplicationGroup {}

                impl<T: Ring + MultiplicationGroup> Field for T {}

                macro_rules! zero_one_integer_impl {
    ($($t: ident)+) => {$(
        impl Zero for $t {
            fn zero() -> Self {
                0
            }
        }

        impl One for $t {
            fn one() -> Self {
                1
            }
        }
    )+};
}

                zero_one_integer_impl!(i128 i64 i32 i16 i8 isize u128 u64 u32 u16 u8 usize);
            }
            pub mod invertible {
                pub trait Invertible {
                    type Output;

                    fn inv(&self) -> Option<Self::Output>;
                }
            }
            pub mod primitive {
                pub trait Primitive<T>: Copy {
                    fn to(self) -> T;
                }

                macro_rules! primitive_one {
    ($t: ident, $($u: ident)+) => {$(
        impl Primitive<$u> for $t {
            fn to(self) -> $u {
                self as $u
            }
        }
    )+};
}

                macro_rules! primitive {
    ($($t: ident)+) => {$(
        primitive_one!($t, u8 u16 u32 u64 u128 usize i8 i16 i32 i64 i128 isize);
    )+}
}

                primitive!(u8 u16 u32 u64 u128 usize i8 i16 i32 i64 i128 isize);
            }
        }
    }
}
fn main() {
    let mut sin = std::io::stdin();
    let input = algo_lib::io::input::Input::new(&mut sin);
    let mut stdout = std::io::stdout();
    let output = algo_lib::io::output::Output::new(&mut stdout);
    solution::run(input, output);
}
