#include <iostream>
#include <fstream>
#include <cassert>
#include <map>
#include <vector>
#include <string>

int main(int argc, char* argv[]) {
    std::ios::sync_with_stdio(false);
    std::cin.tie(nullptr);
    const int32_t argc_modifier =
#ifdef _WIN32
        0
#else
        1
#endif
    ;
    if (argc != 2 + argc_modifier)
        throw std::runtime_error("there should be two arguments: for samples.in and for coordinates.in");
    std::ifstream insamples(argv[1]), incoordinates(argv[2]);
    int32_t t_samples = -1, t_coordinates = -2;
    insamples >> t_samples; incoordinates >> t_coordinates;
    assert(t_samples == t_coordinates);

    std::cout << "outputtemplate := \"%j_%c.ps\";\n";
    std::cout << "u := 1cm;\n";
    std::cout << "r := .18u;\n";
    std::cout << "\n";
    for (int32_t test_index = 1; test_index <= t_samples; ++test_index) {
        std::cout << std::format("beginfig({});\n", test_index);
        int32_t n_samples = -1, n_coordinates = -2;
        insamples >> n_samples; incoordinates >> n_coordinates;
        assert(n_samples == n_coordinates);
        std::cout << "\tnumeric n;\n";
        std::cout << std::format("\tn = {};\n", n_samples);
        for (int32_t point_index = 1; point_index <= n_coordinates; ++point_index) {
            double x, y;
            incoordinates >> x >> y;
            std::cout << std::format("\tz{} = ({:.30}u, {:.30}u);\n", point_index, x, y);
        }
        std::cout << '\n';
        std::map<std::pair<int32_t, int32_t>, int32_t> edge_indices;
        std::vector<std::pair<std::vector<int32_t>, std::vector<int32_t>>> edge_labels;
        std::vector<std::pair<int32_t, int32_t>> map_edges;
        int32_t m_samples;
        insamples >> m_samples;
        for (int32_t edge_index = 1; edge_index <= m_samples; ++edge_index) {
            int32_t u, v, t;
            insamples >> u >> v >> t;
            if (u == v)
                continue;
            std::pair<int32_t, int32_t> edge(std::min(u, v), std::max(u, v));
            if (!edge_indices.contains(edge))
                edge_indices[edge] = edge_indices.size() + 1;
            const int32_t edge_map_index = edge_indices[edge];
            while (edge_map_index - 1 >= edge_labels.size()) {
                edge_labels.emplace_back();
                map_edges.emplace_back();
            }
            (u < v ? edge_labels[edge_map_index - 1].first : edge_labels[edge_map_index - 1].second).push_back(t);
            map_edges.emplace_back(u, v);
            std::cout << std::format("\tdrawarrow z{} - unitvector(z{} - z{}) * r -- z{} + unitvector(z{} - z{}) * r;\n", u, u, v, v, u, v);
        }
        std::cout << '\n';
        for (const auto [edge, edge_map_index] : edge_indices) {
            std::string left_label;
            for (int32_t left_label_index = 0; left_label_index < edge_labels[edge_map_index - 1].first.size(); ++left_label_index) {
                if (left_label_index)
                    left_label += ",";
                left_label += std::to_string(edge_labels[edge_map_index - 1].first[left_label_index]);
            }
            std::string right_label;
            for (int32_t right_label_index = 0; right_label_index < edge_labels[edge_map_index - 1].second.size(); ++right_label_index) {
                if (right_label_index)
                    right_label += ",";
                right_label += std::to_string(edge_labels[edge_map_index - 1].second[right_label_index]);
            }
            const std::string label_delimiter = left_label.empty() || right_label.empty() ? "" : "/";
            const std::string label = left_label + label_delimiter + right_label;
            std::cout << std::format("\tlabel.bot(\"{}\", .5(z{} + z{}));\n", label, edge.first, edge.second);
        }
        std::cout << '\n';
        std::cout << "\tfor i := 1 upto n:\n";
        std::cout << "\t\tfill fullcircle scaled 2r shifted z[i] withcolor white;\n";
        std::cout << "\t\tdraw fullcircle scaled 2r shifted z[i];\n";
        std::cout << "\t\tlabel(decimal(i), z[i]);\n";
        std::cout << "\tendfor;\n";
        std::cout << "\t\n";
        std::cout << "\tcurrentpicture := currentpicture shifted (-llcorner currentpicture);\n";
        std::cout << "endfig;\n";
        std::cout << '\n';
    }
    std::cout << "bye\n";
}
