#include "testlib.h"
#include <numeric>
#include <cassert>

/**
 * Author: Niyaz Nigmatullin
 *
 */

using namespace std;

vector<int> genPerm(int n) {
	vector<int> a(n);
	iota(a.begin(), a.end(), 0);
	shuffle(a.begin(), a.end());
	return a;
}

void print(vector<int> const &a) {
	for (size_t i = 0; i < a.size(); i++) {
		if (i > 0) cout << ' ';
		if (a[i] >= 0)
			cout << a[i] + 1;
		else
			cout << a[i];
	}
	cout << '\n';
}

struct test {
	int n;
	vector<vector<int>> p;
	int m;
	vector<vector<int>> s;
};

int MAXN = 2000;


void gen_while_enough(int max_n, vector<test> &tests, function<test()> const &gen) {
	int total_n = 0;
	while (true) {
		test t = gen();
		total_n += t.n;
		if (total_n > max_n) break;
		tests.push_back(t);
	}
}

test gen_consistent_lists(vector<vector<int>> const &p, int n, int m, double prob) {
	vector<int> used(n);
	vector<int> pUsed = genPerm(n);
	for (int i = 0; i < m; i++) {
		int pos = pUsed[i];
		used[pos] = 1;
	}
	vector<vector<int>> d(2);
	for (int e = 0; e < 2; e++) {
		for (int i = 0; i < n; i++) {
			int value = p[e][i];
			if (used[value]) {
				if (rnd.next() < prob) {
					d[e].push_back(value);
				} else {
					d[e].push_back(-1);
				}
			}
		}
	}
	return test{n, p, m, d};
}

void gen_consistent(int n, int m, double prob, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2);
		p[0] = genPerm(n);
		p[1] = genPerm(n);
		return gen_consistent_lists(p, n, m, prob);
	});
}

vector<int> removeSome(vector<int> const &a, int k, double prob) {
	assert(!a.empty());
	vector<int> res;
	for (int i = 0; i < k; i++) {
		int x = a[i];
		if (rnd.next() < prob) {
			res.push_back(x);
		} else res.push_back(-1);
	}
	return res;
}

void gen_full_random(int n, int m, double prob, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2);
		p[0] = genPerm(n);
		p[1] = genPerm(n);
		vector<vector<int>> d(2);
		d[0] = removeSome(genPerm(n), m, prob);
		d[1] = removeSome(genPerm(n), m, prob);
		return test{n, p, m, d};
	});
}

void gen_same_permutation(int n, int m, double prob, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2);
		p[0] = genPerm(n);
		p[1] = p[0];
		return gen_consistent_lists(p, n, m, prob);
	});
}

void gen_reversed_permutation(int n, int m, double prob, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2);
		p[0] = genPerm(n);
		p[1] = p[0];
		reverse(p[1].begin(), p[1].end());
		return gen_consistent_lists(p, n, m, prob);
	});
}

void gen_special1(int n, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2, vector<int>(n));
		int ones = (n + 2) / 3;
		for (int i = 0; i < ones; i++) {
			p[0][3 * i] = p[1][3 * i] = i;
		}
		vector<vector<int>> pd(2);
		pd[0] = genPerm(n - ones);
		pd[1] = genPerm(n - ones);
		for (int i = 0; i < 2; i++) {
			int current = 0;
			for (int j = 0; j < n; j++) {
				if (j % 3 == 0) continue;
				p[i][j] = pd[i][current++] + ones;
			}
		}
		vector<int> d(ones + ones - 1, -1);
		for (int i = 0; i < ones; i++) {
			d[2 * i] = i;
		}
		return test{n, p, (int) d.size(), {d, d}};
	});
}

vector<int> random_partition(int sum, int count) {
	assert(sum >= 0);
	vector<int> a(count + 1);
	a[0] = 0;
	a[1] = sum;
	for (int i = 2; i <= count; i++) {
		a[i] = rnd.next(0, sum);
	}
	sort(a.begin(), a.end());
	for (int i = 0; i <= count; i++) {
		a[i] = a[i + 1] - a[i];
	}
	a.pop_back();
	return a;
}

void gen_special2(int n, int m, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> dd(2);
		dd[0] = genPerm(m);
		dd[1] = genPerm(m);
		auto gen_perm_partition = [&](vector<int> const &z) {
			vector<int> partition = random_partition(n - 1 - 3 * (m - 1), m - 1);
			vector<int> p(n, -1);
			int current = 0;
			for (int i = 0; i < m; i++) {
				assert(current < n);
				p[current] = z[i];
				if (i + 1 < m) {
					current += partition[i] + 3;
				}
			}
			assert(current == n - 1);
			vector<int> q = genPerm(n - m);
			current = 0;
			for (int i = 0; i < n; i++) {
				if (p[i] < 0) {
					p[i] = q[current++] + m;
				}
			}
			vector<int> d;
			for (int i = 0; i < m; i++) {
				d.push_back(z[i]);
				if (i + 1 < m) {
					for (int j = 0; j < 1 + partition[i]; j++) {
						d.push_back(-1);
					}
				}
			}
			return make_pair(p, d);
		};
		vector<vector<int>> p(2), d(2);
		tie(p[0], d[0]) = gen_perm_partition(dd[0]);
		tie(p[1], d[1]) = gen_perm_partition(dd[1]);
		assert(d[0].size() == d[1].size());
		return test{n, p, (int) d[0].size(), d};
	});
}

void gen_subset(int n, vector<test> &tests) {
	gen_while_enough(MAXN, tests, [&]() {
		vector<vector<int>> p(2);
		p[0] = genPerm(n);
		p[1] = genPerm(n);
		vector<int> f(n);
		for (int i = 0; i < n; i++) {
			f[i] = rnd.next(0, 2);
		}
		vector<vector<int>> d(2);
		for (int j = 0; j < 2; j++) {
			for (int i = 0; i < n; i++) {
				int value = p[j][i];
				if (f[value] == 0) continue;
				if (f[value] == 1) d[j].push_back(-1);
					else d[j].push_back(value);
			}
		}
		assert(d[0].size() == d[1].size());
		return test{n, p, (int) d[0].size(), d};
	});
}

int main(int argc, char **argv) {
	registerGen(argc, argv, 1);
	int n = opt<int>("n");
	int m = opt<int>("m", 0);
	assert(n >= m);
	double prob = opt<double>("p", 0.5);
	if (has_opt("MAXN")) {
		MAXN = opt<int>("MAXN");
	}
	vector<test> tests;
	string type = opt<string>("type", "consistent");
	if (type == "consistent") {
		gen_consistent(n, m, prob, tests);
	} else if (type == "full") {
		gen_full_random(n, m, prob, tests);
	} else if (type == "same_permutation") {
		gen_same_permutation(n, m, prob, tests);
	} else if (type == "reversed_permutation") {
		gen_reversed_permutation(n, m, prob, tests);
	} else if (type == "special1") {
		gen_special1(n, tests);
	} else if (type == "special2") {
		gen_special2(n, m, tests);
	} else if (type == "subset") {
		gen_subset(n, tests);
	} else {
		cerr << "Unknown type: '" << type << "'" << endl;
		assert(false);
	}
	cout << tests.size() << '\n';
	for (auto &t: tests) {
		cout << t.n << '\n';
		print(t.p[0]);
		print(t.p[1]);
		assert(1 <= t.m && t.m <= t.n);
		cout << t.m << '\n';
		print(t.s[0]);
		print(t.s[1]);
	}
}
